@@ -108,6 +108,7 @@ class StoragePool(_StorageObject):
     TYPE_GLUSTER = "gluster"
     TYPE_RBD     = "rbd"
     TYPE_SHEEPDOG = "sheepdog"
+    TYPE_ZFS     = "zfs"
 
     # Pool type descriptions for use in higher level programs
     _descs = {}
@@ -122,6 +123,7 @@ class StoragePool(_StorageObject):
     _descs[TYPE_GLUSTER] = _("Gluster Filesystem")
     _descs[TYPE_RBD]     = _("RADOS Block Device/Ceph")
     _descs[TYPE_SHEEPDOG] = _("Sheepdog Filesystem")
+    _descs[TYPE_ZFS]     = _("ZFS Pool")
 
     @staticmethod
     def get_pool_types():
@@ -158,7 +160,7 @@ class StoragePool(_StorageObject):
 
         try:
             xml = conn.findStoragePoolSources(pool_type, source_xml, 0)
-        except libvirt.libvirtError, e:
+        except libvirt.libvirtError as e:
             if util.is_error_nosupport(e):
                 return []
             raise
@@ -175,16 +177,17 @@ class StoragePool(_StorageObject):
         for source in sources.sources:
             source_xml = source.get_xml_config()
 
-            pool_xml = "<pool>\n%s\n</pool>" % (util.xml_indent(source_xml, 2))
+            pool_xml = "<pool>\n%s\n</pool>" % (
+                XMLBuilder.xml_indent(source_xml, 2))
             parseobj = StoragePool(conn, parsexml=pool_xml)
             parseobj.type = pool_type
 
             obj = StoragePool(conn)
             obj.type = pool_type
             obj.source_path = parseobj.source_path
-            for host in parseobj.hosts:
-                parseobj.remove_host(host)
-                obj.add_host_obj(host)
+            for h in parseobj.hosts:
+                parseobj.remove_host(h)
+                obj.add_host_obj(h)
             obj.source_name = parseobj.source_name
             obj.format = parseobj.format
 
@@ -223,9 +226,8 @@ class StoragePool(_StorageObject):
             defpool.name = name
             defpool.target_path = path
             defpool.install(build=True, create=True, autostart=True)
-            conn.clear_cache(pools=True)
             return defpool
-        except Exception, e:
+        except Exception as e:
             raise RuntimeError(
                 _("Couldn't create default storage pool '%s': %s") %
                 (path, str(e)))
@@ -260,7 +262,7 @@ class StoragePool(_StorageObject):
             for pool in conn.fetch_all_pools():
                 if pool.name == "default":
                     return pool.target_path
-        except:
+        except Exception:
             pass
 
         if build:
@@ -292,9 +294,14 @@ class StoragePool(_StorageObject):
         Finds a name similar (or equal) to passed 'basename' that is not
         in use by another pool. Extra params are passed to generate_name
         """
-        return util.generate_name(basename,
-                                  conn.storagePoolLookupByName,
-                                  **kwargs)
+        def cb(name):
+            for pool in conn.fetch_all_pools():
+                if pool.name == name:
+                    return True
+            return False
+
+        kwargs["lib_collision"] = False
+        return util.generate_name(basename, cb, **kwargs)
 
 
     def __init__(self, *args, **kwargs):
@@ -357,10 +364,11 @@ class StoragePool(_StorageObject):
     source_path = property(_get_source, _set_source)
 
     def _default_source_name(self):
-        if not self.supports_property("source_name"):
-            return None
+        srcname = None
 
-        if self.type == StoragePool.TYPE_NETFS:
+        if not self.supports_property("source_name"):
+            srcname = None
+        elif self.type == StoragePool.TYPE_NETFS:
             srcname = self.name
         elif self.type == StoragePool.TYPE_RBD:
             srcname = "rbd"
@@ -442,12 +450,12 @@ class StoragePool(_StorageObject):
                             self.TYPE_DISK, self.TYPE_ISCSI, self.TYPE_SCSI,
                             self.TYPE_GLUSTER],
             "source_name": [self.TYPE_LOGICAL, self.TYPE_GLUSTER,
-                            self.TYPE_RBD, self.TYPE_SHEEPDOG],
+                            self.TYPE_RBD, self.TYPE_SHEEPDOG, self.TYPE_ZFS],
             "hosts": [self.TYPE_NETFS, self.TYPE_ISCSI, self.TYPE_GLUSTER,
                      self.TYPE_RBD, self.TYPE_SHEEPDOG],
             "format": [self.TYPE_FS, self.TYPE_NETFS, self.TYPE_DISK],
             "iqn": [self.TYPE_ISCSI],
-            "target_path" : [self.TYPE_DIR, self.TYPE_FS, self.TYPE_NETFS,
+            "target_path": [self.TYPE_DIR, self.TYPE_FS, self.TYPE_NETFS,
                              self.TYPE_LOGICAL, self.TYPE_DISK, self.TYPE_ISCSI,
                              self.TYPE_SCSI, self.TYPE_MPATH]
         }
@@ -471,8 +479,22 @@ class StoragePool(_StorageObject):
             StoragePool.TYPE_DIR, StoragePool.TYPE_FS,
             StoragePool.TYPE_NETFS, StoragePool.TYPE_LOGICAL,
             StoragePool.TYPE_DISK,
-            StoragePool.TYPE_RBD, StoragePool.TYPE_SHEEPDOG]
+            StoragePool.TYPE_RBD, StoragePool.TYPE_SHEEPDOG,
+            StoragePool.TYPE_ZFS]
 
+    def get_disk_type(self):
+        if (self.type == StoragePool.TYPE_DISK or
+            self.type == StoragePool.TYPE_LOGICAL or
+            self.type == StoragePool.TYPE_SCSI or
+            self.type == StoragePool.TYPE_MPATH or
+            self.type == StoragePool.TYPE_ZFS):
+            return StorageVolume.TYPE_BLOCK
+        if (self.type == StoragePool.TYPE_GLUSTER or
+            self.type == StoragePool.TYPE_RBD or
+            self.type == StoragePool.TYPE_ISCSI or
+            self.type == StoragePool.TYPE_SHEEPDOG):
+            return StorageVolume.TYPE_NETWORK
+        return StorageVolume.TYPE_FILE
 
     ##################
     # Build routines #
@@ -512,37 +534,39 @@ class StoragePool(_StorageObject):
 
         try:
             pool = self.conn.storagePoolDefineXML(xml, 0)
-        except Exception, e:
+        except Exception as e:
             raise RuntimeError(_("Could not define storage pool: %s") % str(e))
 
         errmsg = None
         if build:
             try:
                 pool.build(libvirt.VIR_STORAGE_POOL_BUILD_NEW)
-            except Exception, e:
+            except Exception as e:
                 errmsg = _("Could not build storage pool: %s") % str(e)
 
         if create and not errmsg:
             try:
                 pool.create(0)
-            except Exception, e:
+            except Exception as e:
                 errmsg = _("Could not start storage pool: %s") % str(e)
 
         if autostart and not errmsg:
             try:
                 pool.setAutostart(True)
-            except Exception, e:
+            except Exception as e:
                 errmsg = _("Could not set pool autostart flag: %s") % str(e)
 
         if errmsg:
             # Try and clean up the leftover pool
             try:
                 pool.undefine()
-            except Exception, e:
+            except Exception as e:
                 logging.debug("Error cleaning up pool after failure: " +
                               "%s" % str(e))
             raise RuntimeError(errmsg)
 
+        self.conn.cache_new_pool(pool)
+
         return pool
 
 
@@ -666,22 +690,6 @@ class StorageVolume(_StorageObject):
             raise ValueError(_("Name '%s' already in use by another volume." %
                                 name))
 
-    def _validate_allocation(self, val):
-        ret = self.is_size_conflict(allocation=val)
-        if ret[0]:
-            raise ValueError(ret[1])
-        elif ret[1]:
-            logging.warn(ret[1])
-        return val
-
-    def _validate_capacity(self, val):
-        ret = self.is_size_conflict(capacity=val)
-        if ret[0]:
-            raise ValueError(ret[1])
-        elif ret[1]:
-            logging.warn(ret[1])
-        return val
-
     def _default_format(self):
         if self.file_type == self.TYPE_FILE:
             return "raw"
@@ -697,14 +705,7 @@ class StorageVolume(_StorageObject):
                 return self.TYPE_DIR
             elif self.type == "network":
                 return self.TYPE_NETWORK
-        if (self._pool_xml.type == StoragePool.TYPE_DISK or
-            self._pool_xml.type == StoragePool.TYPE_LOGICAL):
-            return self.TYPE_BLOCK
-        if (self._pool_xml.type == StoragePool.TYPE_GLUSTER or
-            self._pool_xml.type == StoragePool.TYPE_RBD or
-            self._pool_xml.type == StoragePool.TYPE_SHEEPDOG):
-            return self.TYPE_NETWORK
-        return self.TYPE_FILE
+        return self._pool_xml.get_disk_type()
     file_type = property(_get_vol_type)
 
 
@@ -718,10 +719,8 @@ class StorageVolume(_StorageObject):
 
     type = XMLProperty("./@type")
     key = XMLProperty("./key")
-    capacity = XMLProperty("./capacity", is_int=True,
-                           validate_cb=_validate_capacity)
-    allocation = XMLProperty("./allocation", is_int=True,
-                             validate_cb=_validate_allocation)
+    capacity = XMLProperty("./capacity", is_int=True)
+    allocation = XMLProperty("./allocation", is_int=True)
     format = XMLProperty("./target/format/@type", default_cb=_default_format)
     target_path = XMLProperty("./target/path")
     backing_store = XMLProperty("./backingStore/path")
@@ -795,10 +794,16 @@ class StorageVolume(_StorageObject):
     def validate(self):
         if self._pool_xml.type == StoragePool.TYPE_LOGICAL:
             if self.allocation != self.capacity:
-                logging.warn(_("Sparse logical volumes are not supported, "
+                logging.warning(_("Sparse logical volumes are not supported, "
                                "setting allocation equal to capacity"))
                 self.allocation = self.capacity
 
+        isfatal, errmsg = self.is_size_conflict()
+        if isfatal:
+            raise ValueError(errmsg)
+        if errmsg:
+            logging.warning(errmsg)
+
     def install(self, meter=None):
         """
         Build and install storage volume from xml
@@ -848,7 +853,7 @@ class StorageVolume(_StorageObject):
             logging.debug("Storage volume '%s' install complete.",
                           self.name)
             return vol
-        except Exception, e:
+        except Exception as e:
             logging.debug("Error creating storage volume", exc_info=True)
             raise RuntimeError("Couldn't create storage volume "
                                "'%s': '%s'" % (self.name, str(e)))
@@ -864,7 +869,7 @@ class StorageVolume(_StorageObject):
                     vol = self.pool.storageVolLookupByName(self.name)
                 vol.info()
                 break
-            except:
+            except Exception:
                 if time:  # pylint: disable=using-constant-test
                     # This 'if' check saves some noise from the test suite
                     time.sleep(.2)
@@ -881,7 +886,7 @@ class StorageVolume(_StorageObject):
             time.sleep(1)
 
 
-    def is_size_conflict(self, capacity=None, allocation=None):
+    def is_size_conflict(self):
         """
         Report if requested size exceeds its pool's available amount
 
@@ -890,27 +895,22 @@ class StorageVolume(_StorageObject):
             2. String message if some collision was encountered.
         @rtype: 2 element C{tuple}: (C{bool}, C{str})
         """
-        if capacity is None:
-            capacity = self.capacity
-        if allocation is None:
-            allocation = self.allocation
-
         if not self.pool:
             return (False, "")
 
         # pool info is [pool state, capacity, allocation, available]
         avail = self.pool.info()[3]
-        if allocation > avail:
+        if self.allocation > avail:
             return (True, _("There is not enough free space on the storage "
                             "pool to create the volume. "
-                            "(%d M requested allocation > %d M available)" %
-                            ((allocation / (1024 * 1024)),
-                             (avail / (1024 * 1024)))))
-        elif capacity > avail:
+                            "(%d M requested allocation > %d M available)") %
+                            ((self.allocation / (1024 * 1024)),
+                             (avail / (1024 * 1024))))
+        elif self.capacity > avail:
             return (False, _("The requested volume capacity will exceed the "
                              "available pool space when the volume is fully "
                              "allocated. "
-                             "(%d M requested capacity > %d M available)" %
-                             ((capacity / (1024 * 1024)),
-                              (avail / (1024 * 1024)))))
+                             "(%d M requested capacity > %d M available)") %
+                             ((self.capacity / (1024 * 1024)),
+                              (avail / (1024 * 1024))))
         return (False, "")
